@@ -280,7 +280,7 @@ if_vlanid(__unused const struct interface *ifp)
 }
 
 int
-if_vimaster(__unused const struct dhcpcd_ctx *ctx, __unused const char *ifname)
+if_vimaster(__unused struct dhcpcd_ctx *ctx, __unused const char *ifname)
 {
 
 	return 0;
@@ -955,7 +955,8 @@ if_ifa(struct dhcpcd_ctx *ctx, const struct ifa_msghdr *ifam)
 		    ifam->ifam_type != RTM_NEWADDR)
 			break;
 		memcpy(&sdl, rti_info[RTAX_IFA], sizeof(sdl));
-		dhcpcd_handlehwaddr(ctx, ifp->name, CLLADDR(&sdl),sdl.sdl_alen);
+		dhcpcd_handlehwaddr(ifp, ifp->hwtype,
+		    CLLADDR(&sdl), sdl.sdl_alen);
 		break;
 	}
 #ifdef INET
@@ -1599,9 +1600,9 @@ if_initrt(struct dhcpcd_ctx *ctx, rb_tree_t *routes, int af)
 #ifdef INET
 /* XXX We should fix this to write via the BPF interface. */
 ssize_t
-bpf_send(const struct interface *ifp, __unused int fd, uint16_t protocol,
-    const void *data, size_t len)
+bpf_send(const struct bpf *bpf, uint16_t protocol, const void *data, size_t len)
 {
+	const struct interface *ifp = bpf->bpf_ifp;
 	dlpi_handle_t		dh;
 	dlpi_info_t		di;
 	int			r;
