@@ -54,11 +54,15 @@
 #include "logerr.h"
 #include "privsep.h"
 
+#ifdef HAVE_CAPSICUM
+#include <sys/capsicum.h>
+#endif
+
 static void
 ps_bpf_recvbpf(void *arg)
 {
 	struct ps_process *psp = arg;
-	unsigned int flags;
+	struct bpf *bpf = psp->psp_bpf;
 	uint8_t buf[FRAMELEN_MAX];
 	ssize_t len;
 	struct ps_msghdr psm = {
@@ -66,16 +70,16 @@ ps_bpf_recvbpf(void *arg)
 		.ps_cmd = psp->psp_id.psi_cmd,
 	};
 
+	bpf->bpf_flags &= ~BPF_EOF;
 	/* A BPF read can read more than one filtered packet at time.
 	 * This mechanism allows us to read each packet from the buffer. */
-	flags = 0;
-	while (!(flags & BPF_EOF)) {
-		len = bpf_read(&psp->psp_ifp, psp->psp_work_fd,
-		    buf, sizeof(buf), &flags);
+	while (!(bpf->bpf_flags & BPF_EOF)) {
+		len = bpf_read(bpf, buf, sizeof(buf));
 		if (len == -1)
 			logerr(__func__);
 		if (len == -1 || len == 0)
 			break;
+		psm.ps_flags = bpf->bpf_flags;
 		len = ps_sendpsmdata(psp->psp_ctx, psp->psp_ctx->ps_data_fd,
 		    &psm, buf, (size_t)len);
 		if (len == -1 && errno != ECONNRESET)
@@ -85,55 +89,30 @@ ps_bpf_recvbpf(void *arg)
 	}
 }
 
-#ifdef ARP
-static ssize_t
-ps_bpf_arp_addr(uint8_t cmd, struct ps_process *psp, struct msghdr *msg)
-{
-	struct interface *ifp = &psp->psp_ifp;
-	struct iovec *iov = msg->msg_iov;
-	struct in_addr addr;
-	struct arp_state *astate;
-
-	if (psp == NULL) {
-		errno = ESRCH;
-		return -1;
-	}
-
-	assert(msg->msg_iovlen == 1);
-	assert(iov->iov_len == sizeof(addr));
-	memcpy(&addr, iov->iov_base, sizeof(addr));
-	if (cmd & PS_START) {
-		astate = arp_new(ifp, &addr);
-		if (astate == NULL)
-			return -1;
-	} else if (cmd & PS_DELETE) {
-		astate = arp_find(ifp, &addr);
-		if (astate == NULL) {
-			errno = ESRCH;
-			return -1;
-		}
-		arp_free(astate);
-	} else {
-		errno = EINVAL;
-		return -1;
-	}
-
-	return bpf_arp(ifp, psp->psp_work_fd);
-}
-#endif
-
 static ssize_t
 ps_bpf_recvmsgcb(void *arg, struct ps_msghdr *psm, struct msghdr *msg)
 {
 	struct ps_process *psp = arg;
 	struct iovec *iov = msg->msg_iov;
 
-#ifdef ARP
-	if (psm->ps_cmd & (PS_START | PS_DELETE))
-		return ps_bpf_arp_addr(psm->ps_cmd, psp, msg);
+#ifdef PRIVSEP_DEBUG
+	logerrx("%s: IN cmd %x, psp %p", __func__, psm->ps_cmd, psp);
 #endif
 
-	return bpf_send(&psp->psp_ifp, psp->psp_work_fd, psp->psp_proto,
+	switch(psm->ps_cmd) {
+#ifdef ARP
+	case PS_BPF_ARP:	/* FALLTHROUGH */
+#endif
+	case PS_BPF_BOOTP:
+		break;
+	default:
+		/* IPC failure, we should not be processing any commands
+		 * at this point!/ */
+		errno = EINVAL;
+		return -1;
+	}
+
+	return bpf_send(psp->psp_bpf, psp->psp_proto,
 	    iov->iov_base, iov->iov_len);
 }
 
@@ -152,19 +131,40 @@ ps_bpf_start_bpf(void *arg)
 {
 	struct ps_process *psp = arg;
 	struct dhcpcd_ctx *ctx = psp->psp_ctx;
+	char *addr;
+	struct in_addr *ia = &psp->psp_id.psi_addr.psa_in_addr;
+#ifdef HAVE_CAPSICUM
+	cap_rights_t rights;
 
-	setproctitle("[BPF %s] %s", psp->psp_protostr, psp->psp_ifname);
+	/* We need CAP_IOCTL so we can change the BPF filter when we
+	 * need to. */
+	cap_rights_init(&rights, CAP_READ, CAP_WRITE, CAP_EVENT, CAP_IOCTL);
+#endif
 
+	if (ia->s_addr == INADDR_ANY) {
+		ia = NULL;
+		addr = NULL;
+	} else
+		addr = inet_ntoa(*ia);
+	setproctitle("[BPF %s] %s%s%s", psp->psp_protostr, psp->psp_ifname,
+	    addr != NULL ? " " : "", addr != NULL ? addr : "");
 	ps_freeprocesses(ctx, psp);
 
-	psp->psp_work_fd = bpf_open(&psp->psp_ifp, psp->psp_filter);
-	if (psp->psp_work_fd == -1)
+	psp->psp_bpf = bpf_open(&psp->psp_ifp, psp->psp_filter, ia);
+	if (psp->psp_bpf == NULL)
 		logerr("%s: bpf_open",__func__);
+#ifdef HAVE_CAPSICUM
+	else if (cap_rights_limit(psp->psp_bpf->bpf_fd, &rights) == -1 &&
+	    errno != ENOSYS)
+		logerr("%s: cap_rights_limit", __func__);
+#endif
 	else if (eloop_event_add(ctx->eloop,
-	    psp->psp_work_fd, ps_bpf_recvbpf, psp) == -1)
+	    psp->psp_bpf->bpf_fd, ps_bpf_recvbpf, psp) == -1)
 		logerr("%s: eloop_event_add", __func__);
-	else
+	else {
+		psp->psp_work_fd = psp->psp_bpf->bpf_fd;
 		return 0;
+	}
 
 	eloop_exit(ctx->eloop, EXIT_FAILURE);
 	return -1;
@@ -181,14 +181,13 @@ ps_bpf_signal_bpfcb(int sig, void *arg)
 ssize_t
 ps_bpf_cmd(struct dhcpcd_ctx *ctx, struct ps_msghdr *psm, struct msghdr *msg)
 {
-	uint8_t cmd;
+	uint16_t cmd;
 	struct ps_process *psp;
 	pid_t start;
 	struct iovec *iov = msg->msg_iov;
 	struct interface *ifp;
-	struct ipv4_state *istate;
 
-	cmd = (uint8_t)(psm->ps_cmd & ~(PS_START | PS_STOP));
+	cmd = (uint16_t)(psm->ps_cmd & ~(PS_START | PS_STOP));
 	psp = ps_findprocess(ctx, &psm->ps_id);
 
 #ifdef PRIVSEP_DEBUG
@@ -227,11 +226,6 @@ ps_bpf_cmd(struct dhcpcd_ctx *ctx, struct ps_msghdr *psm, struct msghdr *msg)
 	ifp->options = NULL;
 	memset(ifp->if_data, 0, sizeof(ifp->if_data));
 
-	if ((istate = ipv4_getstate(ifp)) == NULL) {
-		ps_freeprocess(psp);
-		return -1;
-	}
-
 	memcpy(psp->psp_ifname, ifp->name, sizeof(psp->psp_ifname));
 
 	switch (cmd) {
@@ -252,12 +246,21 @@ ps_bpf_cmd(struct dhcpcd_ctx *ctx, struct ps_msghdr *psm, struct msghdr *msg)
 	start = ps_dostart(ctx,
 	    &psp->psp_pid, &psp->psp_fd,
 	    ps_bpf_recvmsg, NULL, psp,
-	    ps_bpf_start_bpf, ps_bpf_signal_bpfcb, PSF_DROPPRIVS);
+	    ps_bpf_start_bpf, ps_bpf_signal_bpfcb,
+	    PSF_DROPPRIVS);
 	switch (start) {
 	case -1:
 		ps_freeprocess(psp);
 		return -1;
 	case 0:
+#ifdef HAVE_CAPSICUM
+		if (cap_enter() == -1 && errno != ENOSYS)
+			logerr("%s: cap_enter", __func__);
+#endif
+#ifdef HAVE_PLEDGE
+		if (pledge("stdio", NULL) == -1)
+			logerr("%s: pledge", __func__);
+#endif
 		break;
 	default:
 #ifdef PRIVSEP_DEBUG
@@ -275,17 +278,21 @@ ps_bpf_dispatch(struct dhcpcd_ctx *ctx,
 {
 	struct iovec *iov = msg->msg_iov;
 	struct interface *ifp;
+	uint8_t *bpf;
+	size_t bpf_len;
 
 	ifp = if_findindex(ctx->ifaces, psm->ps_id.psi_ifindex);
+	bpf = iov->iov_base;
+	bpf_len = iov->iov_len;
 
 	switch (psm->ps_cmd) {
 #ifdef ARP
 	case PS_BPF_ARP:
-		arp_packet(ifp, iov->iov_base, iov->iov_len);
+		arp_packet(ifp, bpf, bpf_len, (unsigned int)psm->ps_flags);
 		break;
 #endif
 	case PS_BPF_BOOTP:
-		dhcp_packet(ifp, iov->iov_base, iov->iov_len);
+		dhcp_packet(ifp, bpf, bpf_len, (unsigned int)psm->ps_flags);
 		break;
 	default:
 		errno = ENOTSUP;
@@ -296,59 +303,48 @@ ps_bpf_dispatch(struct dhcpcd_ctx *ctx,
 }
 
 static ssize_t
-ps_bpf_send(const struct interface *ifp, uint8_t cmd,
-    const void *data, size_t len)
+ps_bpf_send(const struct interface *ifp, const struct in_addr *ia,
+    uint16_t cmd, const void *data, size_t len)
 {
 	struct dhcpcd_ctx *ctx = ifp->ctx;
 	struct ps_msghdr psm = {
 		.ps_cmd = cmd,
 		.ps_id = {
 			.psi_ifindex = ifp->index,
-			.psi_cmd = (uint8_t)(cmd &
-			    ~(PS_START | PS_STOP | PS_DELETE)),
+			.psi_cmd = (uint8_t)(cmd & ~(PS_START | PS_STOP)),
 		},
 	};
 
-	if (psm.ps_id.psi_cmd == PS_BPF_ARP_ADDR)
-		psm.ps_id.psi_cmd = PS_BPF_ARP;
+	if (ia != NULL)
+		psm.ps_id.psi_addr.psa_in_addr = *ia;
 
 	return ps_sendpsmdata(ctx, ctx->ps_root_fd, &psm, data, len);
 }
 
 #ifdef ARP
 ssize_t
-ps_bpf_openarp(const struct interface *ifp)
+ps_bpf_openarp(const struct interface *ifp, const struct in_addr *ia)
 {
 
-	return ps_bpf_send(ifp, PS_BPF_ARP | PS_START, ifp, sizeof(*ifp));
+	assert(ia != NULL);
+	return ps_bpf_send(ifp, ia, PS_BPF_ARP | PS_START,
+	    ifp, sizeof(*ifp));
 }
 
 ssize_t
-ps_bpf_addaddr(const struct interface *ifp, const struct in_addr *addr)
+ps_bpf_closearp(const struct interface *ifp, const struct in_addr *ia)
 {
 
-	return ps_bpf_send(ifp, PS_BPF_ARP_ADDR | PS_START, addr, sizeof(*addr));
+	return ps_bpf_send(ifp, ia, PS_BPF_ARP | PS_STOP, NULL, 0);
 }
 
 ssize_t
-ps_bpf_deladdr(const struct interface *ifp, const struct in_addr *addr)
+ps_bpf_sendarp(const struct interface *ifp, const struct in_addr *ia,
+    const void *data, size_t len)
 {
 
-	return ps_bpf_send(ifp, PS_BPF_ARP_ADDR | PS_DELETE, addr, sizeof(*addr));
-}
-
-ssize_t
-ps_bpf_closearp(const struct interface *ifp)
-{
-
-	return ps_bpf_send(ifp, PS_BPF_ARP | PS_STOP, NULL, 0);
-}
-
-ssize_t
-ps_bpf_sendarp(const struct interface *ifp, const void *data, size_t len)
-{
-
-	return ps_bpf_send(ifp, PS_BPF_ARP, data, len);
+	assert(ia != NULL);
+	return ps_bpf_send(ifp, ia, PS_BPF_ARP, data, len);
 }
 #endif
 
@@ -356,19 +352,20 @@ ssize_t
 ps_bpf_openbootp(const struct interface *ifp)
 {
 
-	return ps_bpf_send(ifp, PS_BPF_BOOTP | PS_START, ifp, sizeof(*ifp));
+	return ps_bpf_send(ifp, NULL, PS_BPF_BOOTP | PS_START,
+	    ifp, sizeof(*ifp));
 }
 
 ssize_t
 ps_bpf_closebootp(const struct interface *ifp)
 {
 
-	return ps_bpf_send(ifp, PS_BPF_BOOTP | PS_STOP, NULL, 0);
+	return ps_bpf_send(ifp, NULL, PS_BPF_BOOTP | PS_STOP, NULL, 0);
 }
 
 ssize_t
 ps_bpf_sendbootp(const struct interface *ifp, const void *data, size_t len)
 {
 
-	return ps_bpf_send(ifp, PS_BPF_BOOTP, data, len);
+	return ps_bpf_send(ifp, NULL, PS_BPF_BOOTP, data, len);
 }
