@@ -98,7 +98,7 @@ ps_root_ioctl6(struct dhcpcd_ctx *ctx, unsigned long request, void *data, size_t
 	if (ps_sendcmd(ctx, ctx->ps_root_fd, PS_IOCTL6,
 	    request, data, len) == -1)
 		return -1;
-	return ps_root_readerror(ctx);
+	return ps_root_readerror(ctx, data, len);
 }
 
 ssize_t
@@ -107,5 +107,5 @@ ps_root_route(struct dhcpcd_ctx *ctx, void *data, size_t len)
 
 	if (ps_sendcmd(ctx, ctx->ps_root_fd, PS_ROUTE, 0, data, len) == -1)
 		return -1;
-	return ps_root_readerror(ctx);
+	return ps_root_readerror(ctx, data, len);
 }
